var _templateObject = _taggedTemplateLiteral([''], ['']);

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _taggedTemplateLiteral(strings, raw) { return Object.freeze(Object.defineProperties(strings, { raw: { value: Object.freeze(raw) } })); }

import React from 'react';
import PropTypes from 'prop-types';
import ReactDOM from 'react-dom';
import classnames from 'classnames';

import isString from 'lodash/isString';
import isArray from 'lodash/isArray';

import rule from 'rockey/rule';
import look from './look';

import createElement from './utils/createElement';

import { ROCKEY_MIXIN_HANDLER_KEY } from './handler';

import { CONTEXT_KEY } from './RockeyThemeProvider';
import RockeyComponent from './RockeyComponent';

var COMPONENT_EXTENDS = 'COMPONENT_EXTENDS';
var DEFINE_COMPONENT_NAME = 'DEFINE_COMPONENT_NAME';
var WAS_CALLED_AS_REACT_COMPONENT = 'WAS_CALLED_AS_REACT_COMPONENT';

var getCallType = function getCallType() {
  if (arguments.length === 1 && isString(arguments.length <= 0 ? undefined : arguments[0])) {
    return DEFINE_COMPONENT_NAME;
  } else if (isArray(arguments.length <= 0 ? undefined : arguments[0])) {
    return COMPONENT_EXTENDS;
  } else {
    return WAS_CALLED_AS_REACT_COMPONENT;
  }
};

var anonymysRockeyCounter = 0;
var childCounter = {};

var createEmtpyCss = function createEmtpyCss(name) {
  var css = rule(_templateObject);
  css.wrapWith(name);
  return css;
};

var getRockeyHoc = function getRockeyHoc() {
  var RockeyHoc = function RockeyHoc(BaseComponent) {
    var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        displayName = _ref.displayName,
        parentName = _ref.parentName,
        css = _ref.css,
        at = _ref.at;

    var name = displayName || 'AnonymysRockey' + ++anonymysRockeyCounter;
    var queuedMixins = null;

    var FlexibleRockeyHoc = function FlexibleRockeyHoc() {
      var CALL_TYPE = getCallType.apply(undefined, arguments);

      switch (CALL_TYPE) {
        case DEFINE_COMPONENT_NAME:
          var childComponentName = arguments.length <= 0 ? undefined : arguments[0];

          return RockeyHoc(BaseComponent, {
            displayName: childComponentName,
            parentName: name,
            css: css,
            at: DEFINE_COMPONENT_NAME
          });

        case COMPONENT_EXTENDS:
          var componentCss = rule.apply(undefined, arguments);
          // componentCss.selector = name;

          if (queuedMixins) {
            componentCss.addMixins(queuedMixins);
          }

          if (css) {
            componentCss.addParent(css);
          }

          // TODO: ?
          var childName = null;

          if (parentName && at !== DEFINE_COMPONENT_NAME) {
            if (!childCounter[parentName]) {
              childCounter[parentName] = 0;
            }

            childName = 'Child' + parentName + '-' + ++childCounter[parentName];
          } else {
            childName = name;
          }

          componentCss.wrapWith(childName);

          return RockeyHoc(BaseComponent, {
            displayName: childName,
            parentName: name,
            css: componentCss,
            at: COMPONENT_EXTENDS
          });

        case WAS_CALLED_AS_REACT_COMPONENT:
          var _ref2 = arguments.length <= 0 ? undefined : arguments[0],
              innerRef = _ref2.innerRef,
              props = _objectWithoutProperties(_ref2, ['innerRef']);

          props.ref = innerRef;

          if (!css) {
            css = createEmtpyCss(name);
            // css.selector = name;
          }

          var rockeyCSSRule = null;
          if (at === DEFINE_COMPONENT_NAME) {
            // wrap with name because Function is used as React comopnent
            // but currect css object === parentCss. line :60
            rockeyCSSRule = createEmtpyCss(name);
            rockeyCSSRule.addParent(css);
            // rockeyCSSRule.selector = name;
          } else {
            rockeyCSSRule = css;
          }

          // collect handler mixins
          var handlers = [];
          // TODO: remove this condition. mixins alwasy should be array
          if (rockeyCSSRule.mixins) {
            Object.keys(rockeyCSSRule.mixins).forEach(function (key) {
              var mixin = rockeyCSSRule.mixins[key];

              if (mixin[ROCKEY_MIXIN_HANDLER_KEY]) {
                handlers.push(mixin);
              }
            });
          }

          return React.createElement(RockeyComponent, {
            rockeyCSSRule: rockeyCSSRule,
            selector: name,
            handlers: handlers,
            Component: BaseComponent,
            componentProps: props
          });
        // if (handlers.length) {
        //   return (
        //     <RockeyComponent
        //       rockeyCSSRule={rockeyCSSRule}
        //       selector={name}
        //       handlers={handlers}
        //       Component={BaseComponent}
        //       componentProps={props}
        //     />
        //   );
        // } else {
        //   return (
        //     <RockeyComponent
        //       rockeyCSSRule={rockeyCSSRule}
        //       selector={name}
        //       Component={BaseComponent}
        //       componentProps={props}
        //     />
        //   );
        // }

        default:
          throw new Error('Wrong component call');
      }
    };

    // FlexibleRockeyHoc.displayName = `Rockey(${name})`;
    FlexibleRockeyHoc.displayName = name;

    FlexibleRockeyHoc.extends = function (displayName, childCss) {
      if (!css) {
        css = createEmtpyCss(name);
      }

      childCss.addParent(css);

      return RockeyHoc(BaseComponent, {
        displayName: displayName,
        parentName: name,
        css: childCss
      });
    };

    FlexibleRockeyHoc.look = function () {
      var component = RockeyHoc(BaseComponent, {
        displayName: displayName,
        parentName: parentName,
        css: css
      });

      var childComponents = look(component, {
        extendBase: true
      }).apply(undefined, arguments);

      Object.keys(childComponents).forEach(function (key) {
        FlexibleRockeyHoc[key] = childComponents[key];
      });

      return childComponents;
    };

    // TODO: create css object instead of queuedMixns
    FlexibleRockeyHoc.addMixins = function (mixins) {
      if (css) {
        css.addMixins(mixins);
      } else {
        queuedMixins = mixins;
      }
    };

    FlexibleRockeyHoc.propTypes = BaseComponent.propTypes;
    FlexibleRockeyHoc.defaultProps = BaseComponent.defaultProps;

    return FlexibleRockeyHoc;
  };

  return RockeyHoc;
};

export { getRockeyHoc };
var RockeyHoc = getRockeyHoc();
export default RockeyHoc;